<?php
namespace KnitPay\Extensions\WPTravelEngine;

use Pronamic\WordPress\Money\Money;
use Pronamic\WordPress\Money\TaxedMoney;
use Pronamic\WordPress\Money\Parser as MoneyParser;
use Pronamic\WordPress\Pay\Payments\PaymentLines;
use Pronamic\WordPress\Pay\Payments\PaymentData as Pay_PaymentData;
use Pronamic\WordPress\Pay\Payments\Item;
use Pronamic\WordPress\Pay\Payments\Items;
use Pronamic\WordPress\Pay\Payments\PaymentDataInterface as Pronamic_Pay_PaymentDataInterface;
use LLMS_Payment_Gateway;
use LLMS_Order;

/**
 * Title: WP Travel Engine extension
 * Description:
 * Copyright: 2020 Knit Pay
 * Company: Knit Pay
 *
 * @author  knitpay
 * @since   1.9
 */
class PaymentData extends Pay_PaymentData {


	/**
	 * The order ID
	 *
	 * @var LLMS_Order
	 */
	private $order;

	/**
	 * Gateway
	 *
	 * @var Gateway
	 */
	private $gateway;

	/**
	 * User data
	 *
	 * @var array
	 */
	private $user_data;

	/**
	 * Constructs and initializes an LearnPress payment data object.
	 *
	 * @param number  $booking_id
	 *            booking id
	 * @param Gateway $gateway
	 *            gateway.
	 */
	public function __construct( $booking_id, $gateway ) {
		$this->gateway    = $gateway;
		$this->booking_id = $booking_id;

		$this->booking_details = get_post_meta( $booking_id, 'wp_travel_engine_booking_setting', true )['place_order']['booking'];
		$this->return_url      = wp_travel_engine_get_booking_confirm_url();

		parent::__construct();
	}

	/**
	 * Get source indicator
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_source()
	 * @return string
	 */
	public function get_source() {
		return 'wp-travel-engine';
	}

	/**
	 * Get source ID.
	 *
	 * @return string
	 */
	public function get_source_id() {
		return $this->booking_id;
	}

	/**
	 * Get title.
	 *
	 * @return string
	 */
	public function get_title() {
		/* translators: %s: order id */
		return sprintf( __( 'Booking #%s', 'pronamic_ideal' ), $this->get_order_id() );
	}

	/**
	 * Get description
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_description()
	 * @return string
	 */
	public function get_description() {
		// TODO
		$description = '';

		if ( '' === $description ) {
			$description = $this->get_title();
		}

		// Replacements.
		$replacements = array(
			'{booking_id}' => $this->get_order_id(),
		);

		return strtr( $description, $replacements );
	}

	/**
	 * Get order ID
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_order_id()
	 * @return string
	 */
	public function get_order_id() {
		return $this->booking_id;
	}

	/**
	 * Get items
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_items()
	 * @return Items
	 */
	/*
	 * public function get_items() {
	 *
	 * $payment_lines = new PaymentLines();
	 * if ( $items = LP()->get_cart()->get_items() ) {
	 * foreach ( $items as $item ) {
	 *
	 * $line = $payment_lines->new_line();
	 *
	 * $line->set_id( $item['item_id'] );
	 * $line->set_name( get_the_title( $item['item_id'] ) );
	 * $line->set_quantity( $item['quantity'] );
	 * $line->set_discount_amount( new Money($item['data']->get_sale_price(), $this->get_currency_alphabetic_code()) );
	 * $line->set_unit_price( new TaxedMoney($item['data']->get_origin_price(), $this->get_currency_alphabetic_code()) );
	 * $line->set_total_amount( new TaxedMoney($item['data']->get_sale_price(), $this->get_currency_alphabetic_code()) );
	 * $line->set_product_url( get_permalink( $item['item_id'] ) );
	 * }
	 * }
	 *
	 * ppp($payment_lines);ddd();
	 * }
	 */
	public function get_items() {   }

	public function get_amount() {
		global $wte_cart;
		$cart_amounts = $wte_cart->get_total();
		$price        = wp_travel_engine_is_cart_partially_payable() ? $cart_amounts['total_partial'] : $cart_amounts['total'];
		$amount       = new TaxedMoney( $price, $this->get_currency_alphabetic_code() );
		return $amount;
	}

	/**
	 * Get currency
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_currency_alphabetic_code()
	 * @return string
	 */
	public function get_currency_alphabetic_code() {
		return wp_travel_engine_get_currency_code();
	}

	/**
	 * Get email.
	 *
	 * @return string
	 */
	public function get_email() {
		return $this->booking_details['email'];
	}

	/**
	 * Get first name.
	 *
	 * @return string
	 */
	public function get_first_name() {
		if ( isset( $this->booking_details['fname'] ) ) {
			return $this->booking_details['fname'];
		}
	}

	/**
	 * Get last name.
	 *
	 * @return string
	 */
	public function get_last_name() {
		if ( isset( $this->booking_details['lname'] ) ) {
			return $this->booking_details['lname'];
		}
	}

	/**
	 * Get customer name.
	 *
	 * @return string
	 */
	public function get_customer_name() {
		return $this->booking_details['fname'] . ' ' . $this->booking_details['lname'];
	}

	/**
	 * Get address.
	 *
	 * @return null|string
	 */
	public function get_address() {
		return $this->booking_details['address'];
	}

	/**
	 * Get city.
	 *
	 * @return null|string
	 */
	public function get_city() {
		return $this->booking_details['city'];
	}

	/**
	 * Get telephone number.
	 *
	 * @return null|string
	 */
	public function get_telephone_number() {
		// TODO
		return $this->order->billing_phone;
	}
}
